# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IDevice",
    "IDeviceSet",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )

from lazr.restful.declarations import export_as_webservice_entry
from lazr.restful.simple import IMultiplePathPartLocation

from lpresults.hardware.interfaces.devicecategory import IHasDeviceCategory
from lpresults.hardware.interfaces.deviceproduct import IHasDeviceProduct
from lpresults.hardware.interfaces.devicesubproduct import (
    IHasDeviceSubproduct,
    )


class IDevice(
    IMultiplePathPartLocation, IHasDeviceProduct, IHasDeviceSubproduct,
    IHasDeviceCategory):

    export_as_webservice_entry()

    bus = Attribute("Device bus")
    product = Attribute("Device product")
    vendor = Attribute("Device vendor")
    subproduct = Attribute("Device subproduct")
    subvendor = Attribute("Device subvendor")
    category = Attribute("Device category")


class IDeviceSet(Interface):

    def get(id):
        """Return the device with the given id.

        If there's no device with the given id, return None.

        :param id: The primary id associated with a device.
        """

    def getByName(
        bus_name, category_name=None, product_name=None,
        vendor_name=None, product_id=None, vendor_id=None,
        subproduct_id=None, subvendor_id=None):
        """Return the device with the given names.

        If there's no device with the given name, create one

        :param bus_name: The name of the bus.
        :param category_name: Optionally, category name.
        :param product_name: Optionally, product name.
        :param vendor_name: Optionally, vendor name.
        :param product_id: Optionally, product identifier.
        :param vendor_id: Optionally, vendor identifier.
        :param subproduct_id: Optionally, subproduct identifier.
        :param subvendor_id: Optionally, subvendor identifier.
        """
