# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IDeviceVendor",
    "IDeviceVendorTarget",
    "IHasDeviceVendor",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import (
    Int,
    TextLine,
    )

from lazr.restful.declarations import exported

from lpresults.hardware.interfaces.devicebus import IHasDeviceBus


class IDeviceVendor(Interface):

    name = Attribute("Device vendor name")


class IHasDeviceVendor(IHasDeviceBus):
    """An entity which has a device vendor."""

    vendor_id = exported(
        Int(
            title=u"Device vendor identifier",
            required=True,
            readonly=True))

    vendor_name = exported(
        TextLine(
            title=u"Device vendor name",
            required=True,
            readonly=True))


class IDeviceVendorTarget(Interface):
    """A target for which device vendors can be created."""

    def getBusId():
        """Get the bus identifier for this target."""

    def getVendorId(name, vendor_id=None):
        """Get the vendor identifier under this target.

        :param name: Vendor name.
        :param vendor_id: Optionally, vendor identifier.
        """

    def getVendor(name, vendor_id=None):
        """Get the vendor object under this target.

        :param name: Vendor name.
        :param vendor_id: Optionally, vendor identifier.
        """
