# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DeviceSubproduct",
    "DeviceSubproductTargetMixin",
    ]

from storm.locals import (
    Int,
    Unicode,
    )

from zope.interface import implements

from lpresults.hardware.interfaces.devicesubproduct import (
    IDeviceSubproduct,
    IDeviceSubproductTarget,
    )
from lpresults.hardware.models.store import get_hardware_store


class DeviceSubproduct:
    """A device subproduct."""
    __storm_table__ = "devicesubproduct"
    __storm_primary__ = (
        "product_id", "vendor_id", "subproduct_id", "subvendor_id",)

    implements(IDeviceSubproduct)

    name = Unicode(allow_none=False)
    product_id = Int(name="product", allow_none=False)
    vendor_id = Int(name="vendor", allow_none=False)
    subproduct_id = Int(name="subproduct", allow_none=False)
    subvendor_id = Int(name="subvendor", allow_none=False)


class DeviceSubproductTargetMixin:

    implements(IDeviceSubproductTarget)

    def getBusId(self):
        """See `IDeviceProductTarget`."""
        raise NotImplementedError()

    def setSubproduct(
        self, name, product_id, vendor_id, subproduct_id, subvendor_id):
        """See `IDeviceSubproductTarget`."""
        bus_id = self.getBusId()
        return get_hardware_store().execute(
            "SELECT set_device_subproduct(?, ?, ?, ?, ?, ?)",
            (name, bus_id, product_id, vendor_id,
            subproduct_id, subvendor_id,), noresult=True)
