# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ProcessorPlatform",
    "HasProcessorPlatformMixin",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Unicode,
    )

from zope.interface import implements

from lpresults.hardware.interfaces.processorplatform import IProcessorPlatform
from lpresults.hardware.models.store import get_hardware_store


class ProcessorPlatform:

    __storm_table__ = "processorplatform"

    implements(IProcessorPlatform)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    name = Unicode(allow_none=False)

    def __init__(self, name):
        self.name = name


class HasProcessorPlatformMixin:

    platform = None

    def _setPlatformName(self, name):
        store = get_hardware_store()
        platform = store.find(ProcessorPlatform, name=name).one()
        if not platform:
            platform = ProcessorPlatform(name)

        self.platform = platform

    def _getPlatformName(self):
        return getattr(self.platform, "name", None)

    platform_name = property(_getPlatformName, _setPlatformName)
