# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "SystemUnit",
    "SystemUnitSet",
    ]

from storm.locals import (
    AutoReload,
    Int,
    List,
    Reference,
    Unicode,
    )

from zope.interface import implements

from lpresults.database.array import ArrayOverlaps
from lpresults.database.constants import UTC_NOW
from lpresults.database.datetime import UTCDateTime
from lpresults.database.search import search_to_expression

from lpresults.hardware.interfaces.system import ISystem
from lpresults.hardware.interfaces.systemunit import (
    ISystemUnit,
    ISystemUnitSet,
    )
from lpresults.hardware.models.devicestate import (
    DeviceState,
    DeviceStateTargetMixin,
    )
from lpresults.hardware.models.processorstate import (
    ProcessorState,
    ProcessorStateTargetMixin,
    )
from lpresults.hardware.models.store import get_hardware_store
from lpresults.hardware.models.system import System
from lpresults.hardware.models.unit import (
    Unit,
    UnitSet,
    )


class SystemUnit(Unit, DeviceStateTargetMixin, ProcessorStateTargetMixin):

    __storm_table__ = "systemunit"

    implements(ISystemUnit)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    date_created = UTCDateTime(allow_none=False, default=UTC_NOW)
    sequence = Int(allow_none=False, default=AutoReload)
    system_id = Int(name="system", allow_none=False)
    identifiers = List(type=Unicode())

    system = Reference(system_id, System.id)

    def __init__(self, system, identifiers, date_created=None):
        super(SystemUnit, self).__init__(identifiers)
        self.system = system

        if date_created is not None:
            self.date_created = date_created

    @property
    def __parent__(self):
        return self.system


class SystemUnitSet(UnitSet):
    """See `ISystemUnitSet`."""

    implements(ISystemUnitSet)

    def new(self, identifiers, date_created=None):
        """See `IUnitSet`."""
        return SystemUnit(self.target, identifiers, date_created)

    def get(self, id):
        """See `ISeriesSet`."""
        return get_hardware_store().get(SystemUnit, id)

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(SystemUnitSet, self).buildSearchExpressions(
            search_params)

        if ISystem.providedBy(self.target):
            expressions.append(SystemUnit.system_id == self.target.id)

        if search_params.sequence is not None:
            expressions.append(
                search_to_expression(
                    SystemUnit.sequence, search_params.sequence))

        if search_params.identifiers is not None:
            expressions.append(ArrayOverlaps(
                SystemUnit.identifiers, search_params.identifiers))

        return expressions

    def getSearchResultTables(self):
        """See SearchSet."""
        return SystemUnit


DeviceState.system_unit = Reference(
    DeviceState.system_unit_id, SystemUnit.id)
ProcessorState.system_unit = Reference(
    ProcessorState.system_unit_id, SystemUnit.id)
