# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestDeviceState",
    ]

from unittest import defaultTestLoader

from lpresults.hardware.testing.cases import TestCaseWithFactory
from lpresults.hardware.testing.layers import FunctionalDatabaseLayer


class TestDeviceState(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_driverName(self):
        state = self.factory.makeDeviceState()
        self.assertEquals(state.driver_name, None)

        driver_name = self.factory.getUniqueString(u"driver")
        state = self.factory.makeDeviceState(driver_name=driver_name)
        self.assertEquals(state.driver_name, driver_name)

    def test_driver(self):
        state = self.factory.makeDeviceState()
        self.assertEquals(state.driver, None)

        driver_name = self.factory.getUniqueString(u"driver")
        state = self.factory.makeDeviceState(driver_name=driver_name)
        self.assertNotEquals(state.driver, None)


class TestDeviceStateTargetMixin:

    layer = FunctionalDatabaseLayer

    def getDeviceStateTarget(self):
        raise NotImplementedError()

    def test_addDeviceState(self):
        target = self.getDeviceStateTarget()
        bus_name = self.factory.getUniqueString(u"bus")
        product_name = self.factory.getUniqueString(u"product")
        state = target.addDeviceState(
            bus_name=bus_name, product_name=product_name)
        self.assertNotEquals(state, None)
        self.assertEquals(state.bus_name, bus_name)
        self.assertEquals(state.product_name, product_name)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
