# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestFactory",
    ]

from zope.component import getUtility

from lpresults.database.store import get_store
from lpresults.database.testing import factories as database_factories

from lpresults.hardware.interfaces.device import IDeviceSet
from lpresults.hardware.interfaces.processor import IProcessorSet
from lpresults.hardware.interfaces.system import ISystemSet


class DatabaseFactory(database_factories.DatabaseFactory):

    def getHardwareStore(self):
        return get_store("hardware")


class TestFactory(DatabaseFactory):

    def makeSystem(self, model=None, make=None, version=None):
        if model is None:
            model = self.getUniqueString(u"model")

        return getUtility(ISystemSet).retrieve(model, make, version=None)

    def makeSystemUnit(self, system=None, identifiers=None):
        if system is None:
            system = self.makeSystem()
        if identifiers is None:
            identifiers = [self.getUniqueString(u"unit-identifer")]

        return system.newUnit(identifiers)

    def makeDevice(
        self, bus_name=None, category_name=None, product_name=None,
        vendor_name=None, product_id=None, vendor_id=None,
        subproduct_id=None, subvendor_id=None):

        if bus_name is None:
            bus_name = self.getUniqueString(u"bus")
        if product_name is None and product_id is None:
            product_name = self.getUniqueString(u"product")

        return getUtility(IDeviceSet).getByName(
            bus_name, category_name, product_name, vendor_name,
            product_id, vendor_id, subproduct_id, subvendor_id)

    def makeDeviceState(
        self, system_unit=None, device=None, driver_name=None,
        date_created=None):

        if system_unit is None:
            system_unit = self.makeSystemUnit()
        if device is None:
            device = self.makeDevice()

        return system_unit.addDeviceState(
            device.bus_name, device.category_name, device.product_name,
            device.vendor_name, device.product_id, device.vendor_id,
            device.subproduct_id, device.subvendor_id, driver_name,
            date_created)

    def makeProcessor(
        self, platform_name=None, make=None, model=None, model_number=None,
        model_version=None, model_revision=None, cache=None, other=None):

        if platform_name is None:
            platform_name = self.getUniqueString(u"platform")
        if make is None:
            make = self.getUniqueString(u"make")
        if model is None:
            model = self.getUniqueString(u"model")
        if model_number is None:
            model_number = unicode(self.getUniqueInteger())
        if model_version is None:
            model_version = unicode(self.getUniqueInteger())
        if model_revision is None:
            model_revision = unicode(self.getUniqueInteger())
        if cache is None:
            cache = self.getUniqueInteger()
        if other is None:
            other = self.getUniqueString(u"other")

        return getUtility(IProcessorSet).getByName(
            platform_name, make, model, model_number, model_version,
            model_revision, cache, other)

    def makeProcessorState(
        self, system_unit=None, processor=None, bogomips=None, speed=None,
        count=None, date_created=None):

        if system_unit is None:
            system_unit = self.makeSystemUnit()
        if processor is None:
            processor = self.makeProcessor()
        if bogomips is None:
            bogomips = self.getUniqueInteger()
        if speed is None:
            speed = self.getUniqueInteger()
        if count is None:
            count = self.getUniqueInteger()

        return system_unit.setProcessorState(
            processor.platform_name, processor.make, processor.model,
            processor.model_number, processor.model_version,
            processor.model_revision, processor.cache, processor.other,
            bogomips, speed, count)
