# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ConfigMixin",
    ]

from optparse import (
    OptionGroup,
    OptionValueError,
    )

from pkg_resources import resource_string

from zope.configuration import xmlconfig

from lpresults.tracker import settings


class ConfigMixin:

    # Config defaults
    default_zcml_file = "site.zcml"
    default_zcml_package = settings.__package__

    def addOptions(self, parser):
        """See Application."""
        super(ConfigMixin, self).addOptions(parser)

        group = OptionGroup(parser, "Config options")
        group.add_option("-z", "--zcml-file",
            default=self.default_zcml_file,
            metavar="FILE",
            help="ZCML file to use as configuration,"
                " defaults to: %default")
        group.add_option("-Z", "--zcml-package",
            default=self.default_zcml_package,
            metavar="PACKAGE",
            help="ZCML package to use as configuration,"
                " defaults to: %default")

        parser.add_option_group(group)

    def parseOptions(self, options, args):
        """See Application."""
        super(ConfigMixin, self).parseOptions(options, args)

        try:
            string = resource_string(options.zcml_package, options.zcml_file)
        except IOError, error:
            raise OptionValueError(str(error))

        xmlconfig.string(string)
