# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "LockMixin",
    ]

import os
import sys

from lpresults.scripts.lock import Lock

from lpresults.tracker import settings


class LockMixin:

    @property
    def lockfile(self):
        """Complete path to the lock file."""
        return os.path.join(
            settings.LPRESULTS_SCRIPTS_LOCKPATH, "%s.lock" % self.name)

    def preProcess(self):
        """See Application."""
        super(LockMixin, self).preProcess()

        self.lock = Lock(self.lockfile)
        if not self.lock.acquire(write=True, block=False):
            self.logger.debug("Lockfile %s in use" % self.lockfile)
            sys.exit(1)

    def postProcess(self, error):
        """See Application."""
        self.lock.release()

        super(LockMixin, self).postProcess(error)
