#!/usr/bin/python

__metaclass__ = type

__all__ = [
    "MultiApplication",
    ]

from multiprocessing import (
    Process,
    Queue,
    )

from lpresults.scripts.application import Application
from lpresults.scripts.functor import Functor


class MultiApplication(Application):

    # Application defaults
    default_concurrency = 10

    def addOptions(self, parser):
        """See Application."""
        super(MultiApplication, self).addOptions(parser)

        parser.add_option("-c", "--concurrency",
            default=self.default_concurrency,
            metavar="NUM",
            type="int",
            help="Number of processes to run at a time,"
                " defaults to %default")

    def parseOptions(self, options, args):
        """See Application."""
        super(MultiApplication, self).parseOptions(options, args)

        self.concurrency = options.concurrency
        self.input_queue = Queue()
        self.output_queue = Queue()

    def processInput(self):
        pass

    def processOutput(self):
        pass

    def processMessages(self, index):
        """Process each message here. Must be defined."""
        raise NotImplementedError()

    def process(self):
        """See Application."""
        if not self.concurrency:
            self.processInput()
            self.processMessages(None)
            self.processOutput()

        else:
            message_process_functor = Functor(
                Process, target=self.processMessages)

            processes = []
            processes.append(Process(target=self.processInput))
            processes.extend([message_process_functor(args=(index,))
                for index in xrange(self.concurrency)])
            processes.append(Process(target=self.processOutput))

            for process in processes:
                process.start()

            try:
                for process in processes:
                    process.join()
            except KeyboardInterrupt:
                for process in processes:
                    process.terminate()
                    process.join()
