# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IHasTestSystemUnit",
    "ITestSystemUnit",
    "ITestSystemUnitSet",
    "ITestSystemUnitTarget",
    ]

from lazr.restful.declarations import (
    call_with,
    export_as_webservice_entry,
    export_write_operation,
    exported,
    operation_parameters,
    REQUEST_USER,
    )
from lazr.restful.fields import Reference
from lazr.restful.interface import copy_field

from lpresults.hardware.interfaces.system import ISystem
from lpresults.hardware.interfaces.systemunit import (
    IHasSystemUnit,
    ISystemUnit,
    ISystemUnitSet,
    )
from lpresults.hardware.interfaces.unit import IUnit

from lpresults.tracker.interfaces.teststat import IHasTestStats


class ITestSystemUnit(ISystemUnit, IHasTestStats):
    """A test system unit.

    This extends the concept of a system unit with test information.
    """
    export_as_webservice_entry(singular_name="system_unit")


class ITestSystemUnitSet(ISystemUnitSet):
    """Interface representing the set of TestSystemUnit."""


class IHasTestSystemUnit(IHasSystemUnit):
    """An entity which has a test system unit."""

    system_unit = exported(
        Reference(
            schema=ITestSystemUnit,
            title=u"System Unit",
            required=False,
            readonly=True))


class ITestSystemUnitTarget(IHasTestSystemUnit):

    @call_with(person=REQUEST_USER)
    @operation_parameters(
        model=copy_field(ISystem["model"]),
        make=copy_field(ISystem["make"]),
        version=copy_field(ISystem["version"]),
        identifiers=copy_field(IUnit["identifiers"]),
        )
    @export_write_operation()
    def setSystemUnit(person, model, make, version, identifiers):
        """Set the system unit on this target.

        :param model: The model of the system.
        :param make: The make of the system.
        :param version: The version of the system.
        :param identifiers: List of unique identifiers unique to the system.
        """
