# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = [
    "create",
    ]

from storm.zope.schema import ZSchema


def create():
    from lpresults.tracker.schema import test as patch_package

    return ZSchema(CREATES, DROPS, DELETES, patch_package)


CREATES = [
"""
GRANT SELECT, INSERT
    ON TABLE patch
    TO lpresults
""",
"""
CREATE TABLE persontwin (
    id INTEGER PRIMARY KEY)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE persontwin
    TO lpresults
""",
"""
CREATE TABLE distroseriestwin (
    id INTEGER PRIMARY KEY,
    distribution INTEGER NOT NULL,
    counter INTEGER DEFAULT 0)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE distroseriestwin
    TO lpresults
""",
"""
CREATE INDEX distroseriestwin__distribution__idx
    ON distroseriestwin (distribution)
""",
"""
CREATE TABLE projectseriestwin (
    id INTEGER PRIMARY KEY,
    project INTEGER NOT NULL,
    counter INTEGER DEFAULT 0)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE projectseriestwin
    TO lpresults
""",
"""
CREATE INDEX projectseriestwin__project__idx
    ON projectseriestwin (project)
""",
"""
CREATE TABLE systemunittwin (
    id INTEGER PRIMARY KEY,
    system INTEGER NOT NULL)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE systemunittwin
    TO lpresults
""",
"""
CREATE INDEX systemunittwin__system__idx
    ON systemunittwin (system)
""",
"""
CREATE TABLE testrun (
    id SERIAL NOT NULL PRIMARY KEY,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
    sequence INTEGER NOT NULL,
    person INTEGER,
    system_unit INTEGER,
    distro_series INTEGER,
    project_series INTEGER,
    pass_count INTEGER NOT NULL DEFAULT 0,
    fail_count INTEGER NOT NULL DEFAULT 0,
    uninitiated_count INTEGER NOT NULL DEFAULT 0,
    unresolved_count INTEGER NOT NULL DEFAULT 0,
    unsupported_count INTEGER NOT NULL DEFAULT 0,
    untested_count INTEGER NOT NULL DEFAULT 0,
    xfail_count INTEGER NOT NULL DEFAULT 0,
    CONSTRAINT valid_target CHECK (
        (distro_series IS NOT NULL) OR (project_series IS NOT NULL)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE testrun
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE testrun_id_seq
    TO lpresults
""",
"""
CREATE INDEX testrun__person__idx
    ON testrun (person)
    WHERE person IS NOT NULL
""",
"""
CREATE INDEX testrun__system_unit__idx
    ON testrun (system_unit)
    WHERE system_unit IS NOT NULL
""",
"""
CREATE UNIQUE INDEX testrun__distro_series__date_created__key
    ON testrun (distro_series, date_created)
    WHERE distro_series IS NOT NULL
""",
"""
CREATE UNIQUE INDEX testrun__distro_series__sequence__key
    ON testrun (distro_series, sequence)
    WHERE distro_series IS NOT NULL
""",
"""
CREATE UNIQUE INDEX testrun__project_series__date_created__key
    ON testrun (project_series, date_created)
    WHERE project_series IS NOT NULL
""",
"""
CREATE UNIQUE INDEX testrun__project_series__sequence__key
    ON testrun (project_series, sequence)
    WHERE project_series IS NOT NULL
""",
"""
CREATE TABLE testcase (
    id SERIAL NOT NULL PRIMARY KEY,
    distro_series INTEGER,
    project_series INTEGER,
    name TEXT NOT NULL,
    CONSTRAINT valid_target CHECK (
        (distro_series IS NOT NULL) OR (project_series IS NOT NULL)))
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE testcase
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE testcase_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX testcase__distro_series__name__key
    ON testcase (distro_series, lower(name))
    WHERE distro_series IS NOT NULL
""",
"""
CREATE UNIQUE INDEX testcase__project_series__name__key
    ON testcase (project_series, lower(name))
    WHERE project_series IS NOT NULL
""",
"""
CREATE TABLE testresult (
    test_case INTEGER NOT NULL,
    status INTEGER NOT NULL DEFAULT 2,  -- UNINITIATED
    output TEXT NOT NULL,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
    date_deleted TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT '3000-01-01',
    CONSTRAINT valid_dates CHECK (date_deleted > date_created))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE testresult
    TO lpresults
""",
"""
CREATE UNIQUE INDEX testresult__test_case__date_created__key
    ON testresult (test_case, date_created)
""",
"""
CREATE INDEX testresult__test_case__idx
    ON testresult (test_case)
""",
"""
CREATE INDEX testresult__date_created__date_deleted__idx
    ON testresult (date_created, date_deleted)
""",
"""
CREATE INDEX
    testresult__test_case__status__output__date_created__date_deleted__idx
    ON testresult (
        test_case, status, decode(md5(output), 'hex'), date_created,
        date_deleted)
""",
"""
CREATE FUNCTION make_plpgsql() RETURNS VOID
LANGUAGE SQL
AS
$$
CREATE LANGUAGE plpgsql;
$$
""",
"""
SELECT
    CASE WHEN EXISTS (
        SELECT 1
        FROM pg_catalog.pg_language
        WHERE lanname='plpgsql')
    THEN
        NULL
    ELSE
        make_plpgsql()
    END
""",
""" 
DROP FUNCTION make_plpgsql()
""",
"""
CREATE FUNCTION get_test_case(
    test_distro_series INTEGER,
    test_project_series INTEGER,
    test_name TEXT)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    test_case_id INTEGER;
BEGIN
    LOOP
        SELECT testcase.id
            INTO test_case_id
            FROM testcase
            WHERE testcase.distro_series
                IS NOT DISTINCT FROM test_distro_series
            AND testcase.project_series
                IS NOT DISTINCT FROM test_project_series
            AND testcase.name = test_name;
        IF FOUND THEN
            RETURN test_case_id;
        END IF;

        BEGIN
            INSERT INTO testcase (
                    distro_series,
                    project_series,
                    name)
                VALUES (
                    test_distro_series,
                    test_project_series,
                    test_name)
                RETURNING id INTO test_case_id;
            RETURN test_case_id;
        EXCEPTION
            WHEN unique_violation THEN
                NULL;  -- do nothing
        END;
    END LOOP;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_test_case(INTEGER, INTEGER, TEXT)
    TO lpresults
""",
"""
CREATE FUNCTION add_test_result(
    testresult_test_case INTEGER,
    testresult_status INTEGER,
    testresult_output TEXT,
    testresult_date_created TIMESTAMP)
    RETURNS BOOLEAN
LANGUAGE plpgsql AS
$$
BEGIN
    LOOP
        BEGIN
            INSERT INTO testresult (
                    test_case,
                    status,
                    output,
                    date_created)
                VALUES (
                    testresult_test_case,
                    testresult_status,
                    testresult_output,
                    testresult_date_created);
            RETURN FOUND;
        EXCEPTION
            WHEN unique_violation THEN
                NULL;  -- do nothing
        END;
    END LOOP;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION add_test_result(INTEGER, INTEGER, TEXT, TIMESTAMP)
    TO lpresults
""",
"""
CREATE FUNCTION testrun_insert_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF NEW.distro_series IS NOT NULL THEN
        UPDATE distroseriestwin SET counter = counter + 1
            WHERE id = NEW.distro_series RETURNING counter INTO NEW.sequence;
    ELSIF NEW.project_series IS NOT NULL THEN
        UPDATE projectseriestwin SET counter = counter + 1
            WHERE id = NEW.project_series RETURNING counter INTO NEW.sequence;
    END IF;

    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testrun_insert_before()
    TO lpresults
""",
"""
CREATE TRIGGER testrun_insert_before_trigger
    BEFORE INSERT ON testrun
    FOR EACH ROW
    EXECUTE PROCEDURE testrun_insert_before()
""",
"""
CREATE FUNCTION testrun_insert_after() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    INSERT INTO testresult (
            test_case,
            status,
            output,
            date_created,
            date_deleted)
        SELECT
                testresult.test_case,
                testresult.status,
                testresult.output,
                MIN(testrun.date_created),
                testresult.date_deleted
            FROM testresult
            JOIN testrun
                ON testrun.date_created > NEW.date_created
                AND testrun.date_created < testresult.date_deleted
            WHERE testresult.date_created < NEW.date_created
            AND CASE WHEN NEW.distro_series IS NOT NULL THEN
                    testrun.distro_series = NEW.distro_series
                ELSE
                    testrun.project_series = NEW.project_series
                END
            GROUP BY
                testresult.test_case,
                testresult.status,
                testresult.output,
                testresult.date_deleted;

    UPDATE testresult
        SET date_deleted = NEW.date_created
        FROM testcase
        WHERE testresult.date_created < NEW.date_created
        AND testresult.date_deleted > NEW.date_created
        AND testresult.test_case = testcase.id
        AND CASE WHEN NEW.distro_series IS NOT NULL THEN
                testcase.distro_series = NEW.distro_series
            ELSE
                testcase.project_series = NEW.project_series
            END;

    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testrun_insert_after()
    TO lpresults
""",
"""
CREATE TRIGGER testrun_insert_after_trigger
    AFTER INSERT ON testrun
    FOR EACH ROW
    EXECUTE PROCEDURE testrun_insert_after()
""",
"""
CREATE FUNCTION testrun_update_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF OLD.distro_series <> NEW.distro_series THEN
        RAISE EXCEPTION 'Cannot update distro_series';
    ELSIF OLD.project_series <> NEW.project_series THEN
        RAISE EXCEPTION 'Cannot update project_series';
    ELSIF OLD.date_created <> NEW.date_created THEN
        RAISE EXCEPTION 'Cannot update project_series';
    ELSE
        RETURN NEW;
    END IF;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testrun_update_before()
    TO lpresults
""",
"""
CREATE TRIGGER testrun_update_before_trigger
    BEFORE UPDATE ON testrun
    FOR EACH ROW
    EXECUTE PROCEDURE testrun_update_before()
""",
"""
CREATE FUNCTION testrun_delete_after() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
DECLARE
    next_testrun RECORD;
BEGIN
    SELECT
        INTO next_testrun
            testrun.date_created
        FROM testrun
        WHERE testrun.date_created > OLD.date_created
        AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                testrun.distro_series = OLD.distro_series
            ELSE
                testrun.project_series = OLD.project_series
            END
        ORDER BY testrun.date_created ASC LIMIT 1;

    -- Between
    IF next_testrun.date_created IS NOT NULL THEN
        UPDATE testresult tr1
            SET date_deleted = tr2.date_deleted
            FROM testresult tr2, testcase tc
            WHERE tr1.test_case = tr2.test_case
            AND tr1.status = tr2.status
            AND decode(md5(tr1.output), 'hex') = decode(md5(tr2.output), 'hex')
            AND tr1.date_deleted = OLD.date_created
            AND tr2.date_created = next_testrun.date_created
            AND tr1.test_case = tc.id
            AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                    tc.distro_series = OLD.distro_series
                ELSE
                    tc.project_series = OLD.project_series
                END;
        IF FOUND THEN
            DELETE FROM testresult tr2
                USING testresult tr1, testcase tc
                WHERE (
                    tr1.test_case = tr2.test_case
                    AND tr1.status = tr2.status
                    AND decode(md5(tr1.output), 'hex')
                        = decode(md5(tr2.output), 'hex')
                    AND tr2.date_created = next_testrun.date_created)
                OR (
                    tr2.date_created = OLD.date_created
                    AND tr2.date_deleted = next_testrun.date_created)
                AND tr1.test_case = tc.id
                AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                        tc.distro_series = OLD.distro_series
                    ELSE
                        tc.project_series = OLD.project_series
                    END;
        ELSE
            DELETE FROM testresult tr
                USING testcase tc
                WHERE tr.date_created = OLD.date_created
                AND tr.date_deleted = next_testrun.date_created
                AND tr.test_case = tc.id
                AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                        tc.distro_series = OLD.distro_series
                    ELSE
                        tc.project_series = OLD.project_series
                    END;
        END IF;

        -- After
        UPDATE testresult tr
            SET date_created = next_testrun.date_created
            FROM testcase
            WHERE tr.date_created = OLD.date_created
            AND tr.test_case = testcase.id
            AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                    testcase.distro_series = OLD.distro_series
                ELSE
                    testcase.project_series = OLD.project_series
                END;
    ELSE
        next_testrun.date_created = '3000-01-01';
        DELETE FROM testresult tr
            USING testcase tc
            WHERE tr.date_created = OLD.date_created
            AND tr.test_case = tc.id
            AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                    tc.distro_series = OLD.distro_series
                ELSE
                    tc.project_series = OLD.project_series
                END;
    END IF;

    -- Before
    UPDATE testresult tr
        SET date_deleted = next_testrun.date_created
        FROM testcase tc
        WHERE tr.date_deleted = OLD.date_created
        AND tr.test_case = tc.id
        AND CASE WHEN OLD.distro_series IS NOT NULL THEN
                tc.distro_series = OLD.distro_series
            ELSE
                tc.project_series = OLD.project_series
            END;

    RETURN OLD;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testrun_delete_after()
    TO lpresults
""",
"""
CREATE TRIGGER testrun_delete_after_trigger
    AFTER DELETE ON testrun
    FOR EACH ROW
    EXECUTE PROCEDURE testrun_delete_after()
""",
"""
CREATE FUNCTION testresult_insert_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
DECLARE
    next_date_created DATE;
    next_testrun DATE;
    previous_date_deleted DATE;
BEGIN
    IF NEW.date_deleted < '3000-01-01' THEN
        RETURN NEW;
    END IF;

    SELECT INTO next_testrun
            CASE WHEN MIN(date_created) > NEW.date_created
            THEN MIN(date_created)
            ELSE MIN(date_deleted)
            END
        FROM testresult tr
        JOIN testcase tc1
            ON tr.test_case = tc1.id
        JOIN testcase tc2
            ON tc1.distro_series IS NOT DISTINCT FROM tc2.distro_series
            AND tc1.project_series IS NOT DISTINCT FROM tc2.project_series
        WHERE date_deleted > NEW.date_created
        AND tc2.id = NEW.test_case;
    IF next_testrun < '3000-01-01' THEN
        -- Set date created of next test result.
        UPDATE testresult
            SET date_created = CASE
                WHEN testresult.status = NEW.status
                    AND testresult.output = NEW.output
                    AND testresult.date_created <> NEW.date_created
                THEN NEW.date_created
                ELSE next_testrun
                END
            WHERE testresult.test_case = NEW.test_case
            AND testresult.date_created <= next_testrun
            AND testresult.date_created >= NEW.date_created
            RETURNING date_created INTO next_date_created;
        IF next_date_created != NEW.date_created THEN
            next_date_created := next_testrun;
            NEW.date_deleted := next_testrun;
        END IF;
    ELSE
        next_date_created := '3000-01-01';
    END IF;

    -- Set date deleted of previous test result.
    UPDATE testresult
        SET date_deleted = CASE
            WHEN testresult.status = NEW.status
                AND testresult.output = NEW.output
            THEN next_date_created
            ELSE NEW.date_created
            END
        WHERE testresult.test_case = NEW.test_case
        AND testresult.date_created <= NEW.date_created
        AND testresult.date_deleted >= NEW.date_created
        RETURNING date_deleted INTO previous_date_deleted;
    IF previous_date_deleted = next_date_created THEN
        IF next_date_created = NEW.date_created THEN
            DELETE FROM testresult
                WHERE test_case = NEW.test_case
                AND testresult.date_created = next_testrun;
        END IF;
        RETURN NULL;
    ELSIF next_date_created = NEW.date_created
          AND previous_date_deleted IS NOT NULL THEN
        RETURN NULL;
    END IF;

    RETURN NEW;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testresult_insert_before()
    TO lpresults
""",
"""
CREATE TRIGGER testresult_insert_before_trigger
    BEFORE INSERT ON testresult
    FOR EACH ROW
    EXECUTE PROCEDURE testresult_insert_before()
""",
"""
CREATE FUNCTION testresult_update_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF OLD.test_case <> NEW.test_case THEN
        RAISE EXCEPTION 'Cannot update test_case';
    ELSIF OLD.status <> NEW.status THEN
        RAISE EXCEPTION 'Cannot update status';
    ELSIF OLD.output <> NEW.output THEN
        RAISE EXCEPTION 'Cannot update output';
    ELSIF NEW.date_deleted = NEW.date_created THEN
        DELETE FROM testresult
            WHERE test_case = OLD.test_case
            AND date_created = OLD.date_created;
        RETURN NULL;
    ELSE
        RETURN NEW;
    END IF;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION testresult_update_before()
    TO lpresults
""",
"""
CREATE TRIGGER testresult_update_before_trigger
    BEFORE UPDATE ON testresult
    FOR EACH ROW
    EXECUTE PROCEDURE testresult_update_before()
""",
"""
ALTER TABLE ONLY testcase
    ADD CONSTRAINT testcase__distro_series__fk
    FOREIGN KEY (distro_series)
    REFERENCES distroseriestwin(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testcase
    ADD CONSTRAINT testcase__project_series__fk
    FOREIGN KEY (project_series)
    REFERENCES projectseriestwin(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testresult
    ADD CONSTRAINT testresult__test_case__fk
    FOREIGN KEY (test_case)
    REFERENCES testcase(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testrun
    ADD CONSTRAINT testrun__distro_series__fk
    FOREIGN KEY (distro_series)
    REFERENCES distroseriestwin(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testrun
    ADD CONSTRAINT testrun__person__fk
    FOREIGN KEY (person)
    REFERENCES persontwin(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testrun
    ADD CONSTRAINT testrun__project_series__fk
    FOREIGN KEY (project_series)
    REFERENCES projectseriestwin(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY testrun
    ADD CONSTRAINT testrun__system_unit__fk
    FOREIGN KEY (system_unit)
    REFERENCES systemunittwin(id) ON DELETE CASCADE
""",
]

DROPS = [
"DROP FUNCTION testresult_update_before()",
"DROP FUNCTION testresult_insert_before()",
"DROP FUNCTION testrun_delete_after()",
"DROP FUNCTION testrun_update_before()",
"DROP FUNCTION testrun_insert_after()",
"DROP FUNCTION testrun_insert_before()",
"DROP FUNCTION add_test_result(INTEGER, INTEGER, TEXT, TIMESTAMP)",
"DROP FUNCTION get_test_case(INTEGER, INTEGER, TEXT)",
"DROP LANGUAGE plpgsql",
"DROP TABLE testresult",
"DROP TABLE testcase",
"DROP TABLE testrun",
"DROP TABLE systemunittwin",
"DROP TABLE projectseriestwin",
"DROP TABLE distroseriestwin",
"DROP TABLE persontwin",
]

DELETES = [
"DELETE FROM testresult",
"DELETE FROM testcase",
"DELETE FROM testrun",
"DELETE FROM systemunittwin",
"DELETE FROM projectseriestwin",
"DELETE FROM distroseriestwin",
"DELETE FROM persontwin",
]
