# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""
Removed sequence functions.
"""

statements = [
"""DROP FUNCTION next_devicebus_product_sequence(INTEGER)""",
"""DROP FUNCTION next_devicebus_vendor_sequence(INTEGER)""",
"""DROP FUNCTION next_system_sequence(INTEGER)""",
"""
CREATE OR REPLACE FUNCTION systemunit_insert_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    UPDATE system SET counter = counter + 1
        WHERE id = NEW.system RETURNING counter INTO NEW.sequence;

    RETURN NEW;
END
$$
""",
"""
CREATE OR REPLACE FUNCTION systemunit_insert_after() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    UPDATE devicestate SET date_deleted = NEW.date_created
        WHERE date_deleted = '3000-01-01'
        AND system_unit = NEW.id;
    UPDATE processorstate SET date_deleted = NEW.date_created
        WHERE date_deleted = '3000-01-01'
        AND system_unit = NEW.id;

    RETURN NEW;
END
$$
""",
"""
CREATE OR REPLACE FUNCTION deviceproduct_product() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        IF NEW.product IS NULL THEN
            UPDATE devicebus SET product_counter = product_counter + 1
                WHERE id = NEW.bus RETURNING product_counter INTO NEW.product;
        ELSIF NEW.product > x'FFFF'::int THEN
            RAISE EXCEPTION 'product must be less than 2^16: %s', NEW.product;
        END IF;

        IF NEW.text_search_vector IS NULL
            AND NOT NEW.name IS NULL THEN
            NEW.text_search_vector := SETWEIGHT(TO_TSVECTOR(
                'simple', text_as_grams(NEW.name)), 'A');
        END IF;
    ELSIF TG_OP = 'UPDATE' THEN
        IF NOT NEW.product = OLD.product THEN
            RAISE EXCEPTION 'product cannot be updated';
        END IF;

        IF NEW.text_search_vector = OLD.text_search_vector
            AND NOT NEW.name = OLD.name THEN
            NEW.text_search_vector := SETWEIGHT(TO_TSVECTOR(
                'simple', text_as_grams(NEW.name)), 'A');
        END IF;
    END IF;
    RETURN NEW;
END
$$
""",
"""
CREATE OR REPLACE FUNCTION devicevendor_vendor() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        IF NEW.vendor IS NULL THEN
            UPDATE devicebus SET vendor_counter = vendor_counter + 1
                WHERE id = NEW.bus RETURNING vendor_counter INTO NEW.vendor;
        ELSIF NEW.vendor > x'FFFF'::int THEN
            RAISE EXCEPTION 'vendor must be less than 2^16: %s', NEW.vendor;
        END IF;

        IF NEW.text_search_vector IS NULL
            AND NOT NEW.name IS NULL THEN
            NEW.text_search_vector := SETWEIGHT(TO_TSVECTOR(
                'simple', text_as_grams(NEW.name)), 'A');
        END IF;
    ELSIF TG_OP = 'UPDATE' THEN
        IF NOT NEW.vendor = OLD.vendor THEN
            RAISE EXCEPTION 'vendor cannot be updated';
        END IF;

        IF NEW.text_search_vector = OLD.text_search_vector
            AND NOT NEW.name = OLD.name THEN
            NEW.text_search_vector := SETWEIGHT(TO_TSVECTOR(
                'simple', text_as_grams(NEW.name)), 'A');
        END IF;
    END IF;
    RETURN NEW;
END
$$
""",
]


def apply(store):
    print __doc__
    for statement in statements:
        store.execute(statement)
