# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.xunit.dispatcher import (
    Dispatcher,
    DispatcherList,
    DispatcherQueue,
    )


class TestDispatcherMixin:

    @property
    def dispatcher_factory(self):
        raise NotImplementedError()

    def test_registerHandler(self):
        values = []

        def handler(a):
            values.append(a)

        dispatcher = self.dispatcher_factory()
        dispatcher.registerHandler("a", handler)
        self.assertEquals(values, [])
        dispatcher.publishEvent("a", 0)
        self.assertEquals(values, [0])

    def test_unregisterHandler(self):
        def handler(a):
            self.fail("Handler should never be called")

        dispatcher = self.dispatcher_factory()
        dispatcher.registerHandler("a", handler)
        dispatcher.unregisterHandler(handler)
        dispatcher.publishEvent("a", 0)

    def test_unregisterListener(self):
        def handler(a):
            self.fail("Handler should never be called")

        dispatcher = self.dispatcher_factory()
        listener = dispatcher.registerHandler("a", handler)
        dispatcher.unregisterListener(listener)
        dispatcher.publishEvent("a", 0)


class TestDispatcher(TestCase, TestDispatcherMixin):

    dispatcher_factory = Dispatcher


class TestDispatcherList(TestCase, TestDispatcherMixin):

    dispatcher_factory = DispatcherList

    def setUp(self):
        self.values = []

    def handler(self, a, b):
        self.values.append((a, b,))

    def test_list(self):
        dispatcher = self.dispatcher_factory()
        dispatcher.registerHandler(("a", "b",), self.handler)
        self.assertEquals(self.values, [])
        dispatcher.publishEvent("a", 0)
        self.assertEquals(self.values, [])
        dispatcher.publishEvent("b", 1)
        self.assertEquals(self.values, [(0, 1,)])

    def test_listOverwrite(self):
        dispatcher = self.dispatcher_factory()
        dispatcher.registerHandler(("a", "b",), self.handler)
        dispatcher.publishEvent("a", 0)
        dispatcher.publishEvent("a", 1)
        dispatcher.publishEvent("b", 1)
        self.assertEquals(self.values, [(1, 1,)])


class TestDispatcherQueue(TestCase, TestDispatcherMixin):

    dispatcher_factory = DispatcherQueue

    def setUp(self):
        self.values = []

    def handler(self, a, b):
        self.values.append((a, b,))

    def test_queue(self):
        dispatcher = self.dispatcher_factory()
        dispatcher.registerHandler(("a", "b",), self.handler)
        self.assertEquals(self.values, [])
        dispatcher.publishEvent("a", 0)
        self.assertEquals(self.values, [])
        dispatcher.publishEvent("a", 1)
        self.assertEquals(self.values, [])
        dispatcher.publishEvent("b", 1)
        self.assertEquals(self.values, [(0, 1,), (1, 1,)])


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
