/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 */
YUI({
    base: '../../../build/yui/',
    filter: 'raw', combine: false, fetchCSS: false
    }).use('test', 'console', 'lpresults.ui.form', function(Y) {

    var TextField = Y.lpresults.ui.TextField,
        TextareaField = Y.lpresults.ui.TextareaField,
        SubmitButton = Y.lpresults.ui.SubmitButton,
        FormButton = Y.lpresults.ui.FormButton,
        Form = Y.lpresults.ui.Form;

    var suite = new Y.Test.Suite('form Tests');

    // Test the TextField.
    suite.add(new Y.Test.Case({
        name: 'TextField',

        setUp: function() {
            this.text = new TextField({
                name: 'field',
                label: 'Field',
                required: true,
                value: 'value',
                hint: 'Some hint'});
        },

        tearDown: function() {
            Y.one('#scaffolding').set('innerHTML', '');
        },

        testRender: function() {
            this.text.render('#scaffolding');
            var contentBox = this.text.get('contentBox'),
                label = contentBox.one('.label'),
                caption = label.one('.caption'),
                separator = label.one('.separator'),
                required = label.one('.required');
            Y.Assert.areEqual('Field', caption.get('text'));
            Y.Assert.areEqual(' ', separator.get('text'));
            Y.Assert.areEqual('(Required)', required.get('text'));
        }
    }));

    suite.add(new Y.Test.Case({
        name: 'TextareaField',

        setUp: function() {
            this.text = new TextareaField(
                {name: 'field',
                 label: 'Field',
                 required: true,
                 value: 'value',
                 hint: 'Some hint'});
        },

        tearDown: function() {
            Y.one('#scaffolding').set('innerHTML', '');
        },

        testRender: function() {
            this.text.render('#scaffolding');
            var contentBox = this.text.get('contentBox'),
                label = contentBox.one('.label'),
                caption = label.one('.caption'),
                separator = label.one('.separator'),
                required = label.one('.required');
            Y.Assert.areEqual('Field', caption.get('text'));
            Y.Assert.areEqual(' ', separator.get('text'));
            Y.Assert.areEqual('(Required)', required.get('text'));
        }
    }));

    suite.add(new Y.Test.Case({
        name: 'FormButton',

        tearDown: function() {
            Y.one('#scaffolding').set('innerHTML', '');
        },

        testRender: function() {
            var field = new FormButton({label: 'Save changes'});
            field.render('#scaffolding');
            var boundingBox = field.get('boundingBox');
            var contentBox = field.get('contentBox');
            var buttonNode = contentBox.one('input');
            Y.Assert.areEqual('span', boundingBox.get('nodeName').toLowerCase());
            Y.Assert.areEqual('save_changes', buttonNode.get('name'));
            Y.Assert.areEqual('Save changes', buttonNode.get('value'));
            Y.Assert.isTrue(buttonNode.hasClass('button'));
        }
    }));

    suite.add(new Y.Test.Case({
        name: 'Form',

        setUp: function() {
            this.text = new TextField({
                name: 'field',
                label: 'Field',
                required: true,
                value: 'value',
                hint: 'Some hint'});
            this.save = new SubmitButton({
                name: 'save',
                value: 'Save'});
            this.quit = new SubmitButton({
                name: 'quit',
                value: 'Quit'});
            this.form = new Form();
        },

        tearDown: function() {
            Y.one('#scaffolding').set('innerHTML', '');
        },

        testAddActionAfterRender: function() {
            this.form.render('#scaffolding');
            this.form.add(this.text);
            this.form.add(this.save);
            this.form.add(this.quit);
            var contentBox = this.form.get('contentBox'),
                actions = contentBox.all('.form-actions input');
            Y.ArrayAssert.itemsAreEqual(['Save', 'Quit'], actions.get('value'));
        },

        testAddActionBeforeRender: function() {
            this.form.add(this.text);
            this.form.add(this.save);
            this.form.add(this.quit);
            this.form.render('#scaffolding');
            var contentBox = this.form.get('contentBox'),
                actions = contentBox.all('.form-actions input');
            Y.ArrayAssert.itemsAreEqual(['Save', 'Quit'], actions.get('value'));
        },

        testSummary: function() {
            this.form.set('summary', 'Nice form');
            this.form.add(this.text);
            this.form.render('#scaffolding');
            var contentBox = this.form.get('contentBox'),
                header = contentBox.one('h2');
            Y.Assert.areEqual('Nice form', header.get('text'));
        }
    }));

    // Lock, stock, and two smoking barrels.
    var handle_complete = function(data) {
        window.status = '::::' + JSON.stringify(data);
        };
    Y.Test.Runner.on('complete', handle_complete);
    Y.Test.Runner.add(suite);

    var console = new Y.Console({newestOnTop: false});
    console.render('#log');

    Y.on('domready', function() {
        Y.Test.Runner.run();
        });
});
